import asyncio

from bleak import BleakClient, BleakScanner
from bleak.backends.device import BLEDevice
from bleak.backends.scanner import AdvertisementData


def simple_callback(device: BLEDevice, advertisement_data: AdvertisementData):
    if device.name != "P2":
        return

    # save the device for later
    global scanner_ble_device
    scanner_ble_device = device

    print(f"Device: {device.name} ({device.address})")

    # create a bytearray from the manufacturer data key
    tmp = list(advertisement_data.manufacturer_data.keys())[0]
    p2AdvData = bytearray(tmp.to_bytes(2, byteorder="little"))
    # extend the bytearray with the manufacturer data value
    p2AdvData.extend(list(advertisement_data.manufacturer_data.values())[0])

    if len(p2AdvData) > 22:  # that means it has a scan response
        print(chr(p2AdvData[22]))

    sn = int(int(p2AdvData[15]) + (int(p2AdvData[16]) << 8))
    print(f"SN: {sn}")

    ldsa = (
        int(p2AdvData[1]) + (int(p2AdvData[2]) << 8) + (int(p2AdvData[3]) << 16)
    ) / 100.0
    print(f"LDSA: {ldsa}")


async def main():
    scanner = BleakScanner(simple_callback, cb=dict(use_bdaddr=True))

    await scanner.start()
    await asyncio.sleep(2.0)
    await scanner.stop()

    if scanner_ble_device is None:
        print("No device found")
        return

    print(scanner_ble_device.name)

    # connect to device
    async with BleakClient(
        scanner_ble_device, disconnected_callback=(lambda: print("disconnected"))
    ) as client:
        print("connected")

        await client.connect()
        print("connected")

        # get the manufacturer data
        manufacturer_data = await client.read_gatt_char(
            "00002a29-0000-1000-8000-00805f9b34fb"
        )
        print(manufacturer_data)


asyncio.run(main())



import base64
import datetime
import time

import pandas as pd
import requests

from naneos.partector.partector1 import Partector1
from naneos.partector.scan_for_partector import scan_for_serial_partectors
from naneos.protobuf.protobuf import create_Combined_entry, create_Partector1_entry


def main():
    devs = scan_for_serial_partectors()
    devs = devs["P1"]

    p1 = Partector1(list(devs.values())[0])

    # every 5 seconds interuptable by keyboard
    while True:
        try:
            time.sleep(20)
            df1 = p1.get_data_pandas()

            abs_time = int(datetime.datetime.now().timestamp())
            p1_device = create_Partector1_entry(df1, p1._serial_number, abs_time)
            devices = [p1_device]

            combined = create_Combined_entry(devices=devices, abs_time=abs_time)

            proto_string = combined.SerializeToString()
            proto_string_base64 = base64.b64encode(proto_string)
            print(proto_string_base64)

            # send this string to the server
            url = "https://hg3zkburji.execute-api.eu-central-1.amazonaws.com/prod/proto/v1"
            headers = {"Content-Type": "application/json", "Accept": "application/json"}
            body = f"""
                    {{
                        "gateway": "python_webhook",
                        "data": "{proto_string_base64.decode()}",
                        "published_at": "{datetime.datetime.now().isoformat()}"
                    }}
                    """

            print(body)

            # send the data to the server
            r = requests.post(url, headers=headers, data=body)
            print(r.status_code)
            print(r.text)

        except KeyboardInterrupt:
            break

    p1.close()


if __name__ == "__main__":
    main()



import logging


class CustomFormatter(logging.Formatter):
    grey = "\x1b[38;20m"
    green = "\x1b[32;20m"
    yellow = "\x1b[33;20m"
    red = "\x1b[31;20m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"
    format = (
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s (%(filename)s:%(lineno)d)"
    )

    FORMATS = {
        logging.DEBUG: grey + format + reset,
        logging.INFO: green + format + reset,
        logging.WARNING: yellow + format + reset,
        logging.ERROR: red + format + reset,
        logging.CRITICAL: bold_red + format + reset,
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


def get_naneos_logger(name: str, level: int = logging.INFO):
    logger = logging.getLogger(name)
    logger.setLevel(level)

    formatter_file = logging.Formatter("%(asctime)s - %(filename)s - %(message)s")
    file_handler = logging.FileHandler("naneos-devices.log")
    file_handler.setLevel(level)
    file_handler.setFormatter(formatter_file)

    formatter_stream = CustomFormatter("%(asctime)s - %(filename)s - %(message)s")
    stream_handler = logging.StreamHandler()
    stream_handler.setLevel(level)
    stream_handler.setFormatter(formatter_stream)

    logger.addHandler(stream_handler)
    logger.addHandler(file_handler)

    return logger



from .download.downloader import download_from_iotweb
from .iotweb import P1uploadThread
from .partector2_pro_garage_upload import Partector2ProGarageUpload

__all__ = ["P1uploadThread", "Partector2ProGarageUpload"]



import time

from naneos.partector import Partector2ProGarage

if __name__ == "__main__":
    p2 = Partector2ProGarage("/dev/tty.usbmodemDOSEMet_1")

    print(p2.write_line("v?", 1))
    time.sleep(2)
    print(p2.get_data_pandas()["T"])
    p2.close()



import time

from naneos.partector import Partector1, Partector2, scan_for_serial_partectors

# Lists all available Partector2 devices
x = scan_for_serial_partectors()

# Split dictionary into P1 and P2 devices
p1_devs = x["P1"]
p2_devs = x["P2"]

if len(p1_devs) > 0:
    print("Found Partector1 devices:")
    for k, v in p1_devs.items():
        print(f"Serial number: {k}, Port: {v}")

    # Connect to the first device
    myP1 = Partector1(list(p1_devs.values())[0], 1)
    time.sleep(2)

    # Get the data as a pandas DataFrame
    data = myP1.get_data_pandas()
    print(data)

    myP1.close()

if len(p2_devs) > 0:
    print("Found Partector2 devices:")
    for k, v in p2_devs.items():
        print(f"Serial number: {k}, Port: {v}")

    # Connect to the first device
    myP2 = Partector2(list(p2_devs.values())[0], 1)
    time.sleep(2)

    # Get the data as a pandas DataFrame
    data = myP2.get_data_pandas()
    print(data)

    myP2.close()



from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class CombinedData(_message.Message):
    __slots__ = ("abs_timestamp", "devices", "gateway_points", "position_points", "wind_points")
    ABS_TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    DEVICES_FIELD_NUMBER: _ClassVar[int]
    GATEWAY_POINTS_FIELD_NUMBER: _ClassVar[int]
    POSITION_POINTS_FIELD_NUMBER: _ClassVar[int]
    WIND_POINTS_FIELD_NUMBER: _ClassVar[int]
    abs_timestamp: int
    devices: _containers.RepeatedCompositeFieldContainer[Device]
    gateway_points: _containers.RepeatedCompositeFieldContainer[GatewayPoint]
    position_points: _containers.RepeatedCompositeFieldContainer[PositionPoint]
    wind_points: _containers.RepeatedCompositeFieldContainer[WindPoint]
    def __init__(self, abs_timestamp: _Optional[int] = ..., devices: _Optional[_Iterable[_Union[Device, _Mapping]]] = ..., gateway_points: _Optional[_Iterable[_Union[GatewayPoint, _Mapping]]] = ..., position_points: _Optional[_Iterable[_Union[PositionPoint, _Mapping]]] = ..., wind_points: _Optional[_Iterable[_Union[WindPoint, _Mapping]]] = ...) -> None: ...

class Device(_message.Message):
    __slots__ = ("type", "serial_number", "device_points")
    TYPE_FIELD_NUMBER: _ClassVar[int]
    SERIAL_NUMBER_FIELD_NUMBER: _ClassVar[int]
    DEVICE_POINTS_FIELD_NUMBER: _ClassVar[int]
    type: int
    serial_number: int
    device_points: _containers.RepeatedCompositeFieldContainer[DevicePoint]
    def __init__(self, type: _Optional[int] = ..., serial_number: _Optional[int] = ..., device_points: _Optional[_Iterable[_Union[DevicePoint, _Mapping]]] = ...) -> None: ...

class DevicePoint(_message.Message):
    __slots__ = ("timestamp", "device_status", "ldsa", "average_particle_diameter", "particle_number_concentration", "temperature", "relative_humidity", "battery_voltage", "particle_mass", "corona_voltage", "diffusion_current", "deposition_voltage", "flow", "ambient_pressure", "electrometer_offset", "electrometer_2_offset", "electrometer_gain", "electrometer_2_gain", "diffusion_current_offset", "particle_number_10nm", "particle_number_16nm", "particle_number_26nm", "particle_number_43nm", "particle_number_70nm", "particle_number_114nm", "particle_number_185nm", "particle_number_300nm", "surface", "sigma_size_dist", "steps_inversion", "current_dist_0", "current_dist_1", "current_dist_2", "current_dist_3", "current_dist_4", "pump_current", "pump_pwm", "cs_status")
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    DEVICE_STATUS_FIELD_NUMBER: _ClassVar[int]
    LDSA_FIELD_NUMBER: _ClassVar[int]
    AVERAGE_PARTICLE_DIAMETER_FIELD_NUMBER: _ClassVar[int]
    PARTICLE_NUMBER_CONCENTRATION_FIELD_NUMBER: _ClassVar[int]
    TEMPERATURE_FIELD_NUMBER: _ClassVar[int]
    RELATIVE_HUMIDITY_FIELD_NUMBER: _ClassVar[int]
    BATTERY_VOLTAGE_FIELD_NUMBER: _ClassVar[int]
    PARTICLE_MASS_FIELD_NUMBER: _ClassVar[int]
    CORONA_VOLTAGE_FIELD_NUMBER: _ClassVar[int]
    DIFFUSION_CURRENT_FIELD_NUMBER: _ClassVar[int]
    DEPOSITION_VOLTAGE_FIELD_NUMBER: _ClassVar[int]
    FLOW_FIELD_NUMBER: _ClassVar[int]
    AMBIENT_PRESSURE_FIELD_NUMBER: _ClassVar[int]
    ELECTROMETER_OFFSET_FIELD_NUMBER: _ClassVar[int]
    ELECTROMETER_2_OFFSET_FIELD_NUMBER: _ClassVar[int]
    ELECTROMETER_GAIN_FIELD_NUMBER: _ClassVar[int]
    ELECTROMETER_2_GAIN_FIELD_NUMBER: _ClassVar[int]
    DIFFUSION_CURRENT_OFFSET_FIELD_NUMBER: _ClassVar[int]
    PARTICLE_NUMBER_10NM_FIELD_NUMBER: _ClassVar[int]
    PARTICLE_NUMBER_16NM_FIELD_NUMBER: _ClassVar[int]
    PARTICLE_NUMBER_26NM_FIELD_NUMBER: _ClassVar[int]
    PARTICLE_NUMBER_43NM_FIELD_NUMBER: _ClassVar[int]
    PARTICLE_NUMBER_70NM_FIELD_NUMBER: _ClassVar[int]
    PARTICLE_NUMBER_114NM_FIELD_NUMBER: _ClassVar[int]
    PARTICLE_NUMBER_185NM_FIELD_NUMBER: _ClassVar[int]
    PARTICLE_NUMBER_300NM_FIELD_NUMBER: _ClassVar[int]
    SURFACE_FIELD_NUMBER: _ClassVar[int]
    SIGMA_SIZE_DIST_FIELD_NUMBER: _ClassVar[int]
    STEPS_INVERSION_FIELD_NUMBER: _ClassVar[int]
    CURRENT_DIST_0_FIELD_NUMBER: _ClassVar[int]
    CURRENT_DIST_1_FIELD_NUMBER: _ClassVar[int]
    CURRENT_DIST_2_FIELD_NUMBER: _ClassVar[int]
    CURRENT_DIST_3_FIELD_NUMBER: _ClassVar[int]
    CURRENT_DIST_4_FIELD_NUMBER: _ClassVar[int]
    PUMP_CURRENT_FIELD_NUMBER: _ClassVar[int]
    PUMP_PWM_FIELD_NUMBER: _ClassVar[int]
    CS_STATUS_FIELD_NUMBER: _ClassVar[int]
    timestamp: int
    device_status: int
    ldsa: int
    average_particle_diameter: int
    particle_number_concentration: int
    temperature: int
    relative_humidity: int
    battery_voltage: int
    particle_mass: int
    corona_voltage: int
    diffusion_current: int
    deposition_voltage: int
    flow: int
    ambient_pressure: int
    electrometer_offset: int
    electrometer_2_offset: int
    electrometer_gain: int
    electrometer_2_gain: int
    diffusion_current_offset: int
    particle_number_10nm: int
    particle_number_16nm: int
    particle_number_26nm: int
    particle_number_43nm: int
    particle_number_70nm: int
    particle_number_114nm: int
    particle_number_185nm: int
    particle_number_300nm: int
    surface: int
    sigma_size_dist: int
    steps_inversion: int
    current_dist_0: int
    current_dist_1: int
    current_dist_2: int
    current_dist_3: int
    current_dist_4: int
    pump_current: int
    pump_pwm: int
    cs_status: int
    def __init__(self, timestamp: _Optional[int] = ..., device_status: _Optional[int] = ..., ldsa: _Optional[int] = ..., average_particle_diameter: _Optional[int] = ..., particle_number_concentration: _Optional[int] = ..., temperature: _Optional[int] = ..., relative_humidity: _Optional[int] = ..., battery_voltage: _Optional[int] = ..., particle_mass: _Optional[int] = ..., corona_voltage: _Optional[int] = ..., diffusion_current: _Optional[int] = ..., deposition_voltage: _Optional[int] = ..., flow: _Optional[int] = ..., ambient_pressure: _Optional[int] = ..., electrometer_offset: _Optional[int] = ..., electrometer_2_offset: _Optional[int] = ..., electrometer_gain: _Optional[int] = ..., electrometer_2_gain: _Optional[int] = ..., diffusion_current_offset: _Optional[int] = ..., particle_number_10nm: _Optional[int] = ..., particle_number_16nm: _Optional[int] = ..., particle_number_26nm: _Optional[int] = ..., particle_number_43nm: _Optional[int] = ..., particle_number_70nm: _Optional[int] = ..., particle_number_114nm: _Optional[int] = ..., particle_number_185nm: _Optional[int] = ..., particle_number_300nm: _Optional[int] = ..., surface: _Optional[int] = ..., sigma_size_dist: _Optional[int] = ..., steps_inversion: _Optional[int] = ..., current_dist_0: _Optional[int] = ..., current_dist_1: _Optional[int] = ..., current_dist_2: _Optional[int] = ..., current_dist_3: _Optional[int] = ..., current_dist_4: _Optional[int] = ..., pump_current: _Optional[int] = ..., pump_pwm: _Optional[int] = ..., cs_status: _Optional[int] = ...) -> None: ...

class GatewayPoint(_message.Message):
    __slots__ = ("timestamp", "serial_number", "firmware_version", "free_memory", "free_heap", "largest_free_block_heap", "cellular_signal", "battery_int_soc", "battery_ext_soc", "battery_ext_voltage", "charging_ext_voltage")
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    SERIAL_NUMBER_FIELD_NUMBER: _ClassVar[int]
    FIRMWARE_VERSION_FIELD_NUMBER: _ClassVar[int]
    FREE_MEMORY_FIELD_NUMBER: _ClassVar[int]
    FREE_HEAP_FIELD_NUMBER: _ClassVar[int]
    LARGEST_FREE_BLOCK_HEAP_FIELD_NUMBER: _ClassVar[int]
    CELLULAR_SIGNAL_FIELD_NUMBER: _ClassVar[int]
    BATTERY_INT_SOC_FIELD_NUMBER: _ClassVar[int]
    BATTERY_EXT_SOC_FIELD_NUMBER: _ClassVar[int]
    BATTERY_EXT_VOLTAGE_FIELD_NUMBER: _ClassVar[int]
    CHARGING_EXT_VOLTAGE_FIELD_NUMBER: _ClassVar[int]
    timestamp: int
    serial_number: int
    firmware_version: int
    free_memory: int
    free_heap: int
    largest_free_block_heap: int
    cellular_signal: int
    battery_int_soc: int
    battery_ext_soc: int
    battery_ext_voltage: int
    charging_ext_voltage: int
    def __init__(self, timestamp: _Optional[int] = ..., serial_number: _Optional[int] = ..., firmware_version: _Optional[int] = ..., free_memory: _Optional[int] = ..., free_heap: _Optional[int] = ..., largest_free_block_heap: _Optional[int] = ..., cellular_signal: _Optional[int] = ..., battery_int_soc: _Optional[int] = ..., battery_ext_soc: _Optional[int] = ..., battery_ext_voltage: _Optional[int] = ..., charging_ext_voltage: _Optional[int] = ...) -> None: ...

class PositionPoint(_message.Message):
    __slots__ = ("timestamp", "latitude", "longitude")
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    LATITUDE_FIELD_NUMBER: _ClassVar[int]
    LONGITUDE_FIELD_NUMBER: _ClassVar[int]
    timestamp: int
    latitude: float
    longitude: float
    def __init__(self, timestamp: _Optional[int] = ..., latitude: _Optional[float] = ..., longitude: _Optional[float] = ...) -> None: ...

class WindPoint(_message.Message):
    __slots__ = ("timestamp", "wind_speed", "wind_angle")
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    WIND_SPEED_FIELD_NUMBER: _ClassVar[int]
    WIND_ANGLE_FIELD_NUMBER: _ClassVar[int]
    timestamp: int
    wind_speed: int
    wind_angle: int
    def __init__(self, timestamp: _Optional[int] = ..., wind_speed: _Optional[int] = ..., wind_angle: _Optional[int] = ...) -> None: ...



import time
from abc import ABC, abstractmethod
from datetime import datetime, timezone
from queue import Queue
from threading import Event, Thread

import pandas
import serial

from naneos.logger.custom_logger import get_naneos_logger
from naneos.partector.blueprints._partector_defaults import PartectorDefaults
from naneos.partector.blueprints._partectorCheckerThread import PartectorCheckerThread

logger = get_naneos_logger(__name__)


class PartectorBluePrint(Thread, PartectorDefaults, ABC):
    """
    Class with the basic functionality of every Partector.
    Mandatory device specific methods are defined abstract and have to be implemented in the child class.
    """

    def __init__(self, serial_number: int = None, port: str = None, verb_freq: int = 1):
        """Initializes the Partector2 and starts the reading thread."""
        self._init(serial_number, port, verb_freq)

    def close(self, blocking: bool = False):
        """Closes the serial connection and stops the reading thread."""
        self._close(blocking)

    def run(self):
        """Thread method. Reads the serial port and puts the data into the queue."""
        checker_thread = PartectorCheckerThread(self)

        while not self.thread_event.is_set():
            self._run()

        checker_thread.stop()
        checker_thread.join()

        if self._ser.isOpen():
            self._ser.close()

    #########################################
    ### Abstract methods
    def set_verbose_freq(self, freq: int):
        """
        Sets the verbose frequency of the device.
        This differs for P1, P2 and P2Pro.
        """
        if not self._connected:
            return

        self._set_verbose_freq(freq)

    @abstractmethod
    def _set_verbose_freq(self, freq: int):
        """
        Sets the verbose frequency of the device.
        This differs for P1, P2 and P2Pro.
        """
        pass

    #########################################
    ### User accessible getters
    def get_serial_number(self) -> int:
        """Gets the serial number via command from the device."""
        return self._serial_wrapper(self._get_serial_number)

    def get_firmware_version(self) -> str:
        """Gets the firmware version via command from the device."""
        return self._serial_wrapper(self._get_firmware_version)

    def write_line(self, line: str, number_of_elem: int = 1) -> list:
        """
        Writes a custom line to the device and returns the tab-separated response as a list.

        Args:
            line (str): The line to write to the device.
            number_of_elem (int, optional): The number of elements in the response. This will be checked. Defaults to 1.

        Returns:
            list: The response as a list.
        """
        self.custom_info_str = line
        self.custom_info_size = number_of_elem + 1

        return self._serial_wrapper(self._custom_info)

    #########################################
    ### User accessible data methods
    def clear_data_cache(self):
        """Clears the data cache."""
        self._queue.queue.clear()

    def get_data_list(self) -> list:
        """Returns the cache as list with timestamp as first element."""
        data_casted = []
        data = list(self._queue.queue)
        self.clear_data_cache()

        for line in data:
            try:
                data_casted.append(self._cast_splitted_input_string(line))
            except Exception as excep:
                logger.warning(f"Could not cast data: {excep}")
                logger.warning(f"Data: {line}")

        return data_casted

    def get_data_pandas(self, data=None) -> pandas.DataFrame:
        """Returns the cache as pandas DataFrame with timestamp as index."""
        if not data:
            data = self.get_data_list()

        columns = self._data_structure.keys()
        df = pandas.DataFrame(data, columns=columns).set_index("dateTime")
        return df

    #########################################
    ### Serial methods (private)
    def _close(self, blocking):
        try:
            self.set_verbose_freq(0)
        except Exception:
            pass
        self.thread_event.set()
        if blocking:
            self.join()

    def _run(self):
        if not self._connected:
            return

        try:
            if self._ser.isOpen():
                self._serial_reading_routine()
        except Exception as e:
            logger.warning(f"Exception occured during threaded serial reading: {e}")

    def _run_check_connection(self) -> bool:
        """Checks if the device is still connected."""
        if not self._connected:
            self._init_serial(self._serial_number, self._port)
            self.set_verbose_freq(1)
        elif self._check_device_connection() is False:
            self._ser.close()
            self._connected = False
            logger.warning(f"Partector on port {self._port} disconnected!")
            self._port = None

        return self._connected

    def _serial_reading_routine(self):
        if not self._connected:
            return

        line = self._read_line()

        if not line or line == "":
            return

        data = [datetime.now(tz=timezone.utc)] + line.split("\t")

        if len(data) == len(self._data_structure):
            if self._queue.full():
                self._queue.get()
            self._queue.put(data)
        else:
            if self._queue_info.full():
                self._queue_info.get()
            self._queue_info.put(data)

    def _check_device_connection(self) -> bool:
        if self.thread_event.is_set() or not self._ser.isOpen():
            return False

        try:
            sn = self._get_serial_number_secure()
            if sn == self._serial_number:
                return True

        except Exception as e:
            logger.error(f"Exception occured during device connection check: {e}")

        return False

    def _check_serial_connection(self):
        """Tries to reopen a closed connection. Raises exceptions on failure."""
        try:
            for _ in range(3):
                self._ser.open() if not self._ser.isOpen() else None
                if self._ser.isOpen():
                    return None
        except Exception as e:
            self._ser.close()
            self._connected = False
            raise ConnectionAbortedError(f"Serial connection aborted: {e}")

    def _serial_wrapper(self, func):
        """Wraps user func in try-except block. Forwards exceptions to the user."""
        if not self._connected:
            return

        for _ in range(self.SERIAL_RETRIES):
            try:
                return func()
            except Exception as e:
                logger.error(f"Exception in _serial_wrapper: {e}")
                excep = f"Exception occured during user function call: {e}"
        raise Exception(excep)

    def _write_line(self, line: str):
        if not self._connected:
            return

        self._check_serial_connection()
        self._ser.write(line.encode())

    def _read_line(self) -> str:
        if not self._connected:
            return ""

        self._check_serial_connection()
        try:
            data = self._ser.readline().decode()
        except Exception as e:
            self._ser.close()
            self._connected = False
            raise Exception(f"Was not able to read from the Serial connection: {e}")
        return data.replace("\r", "").replace("\n", "").replace("\x00", "")

    def _get_and_check_info(self, expected_length: int = 2) -> list:
        """
        Get information from the queue and check its length.

        Parameters:
            expected_length (int): The expected length of the information.

        Returns:
            list: The information from the queue.

        Raises:
            ValueError: If the length of the information does not match the expected length.
        """
        info_data = self._queue_info.get(timeout=self.SERIAL_TIMEOUT_INFO)
        if len(info_data) != expected_length:
            error_msg = f"Received data of length {len(info_data)}, expected {expected_length}. Data: {info_data}"
            raise ValueError(error_msg)
        return info_data

    def _get_serial_number_secure(self) -> int:
        if not self._connected:
            return

        for _ in range(3):
            serial_numbers = [self.get_serial_number() for _ in range(3)]
            if all(x == serial_numbers[0] for x in serial_numbers):
                return serial_numbers[0]
        raise Exception("Was not able to fetch the serial number (secure)!")

    def _get_serial_number(self) -> int:
        self._queue_info.queue.clear()
        self._write_line("N?")
        return int(self._get_and_check_info()[1])

    def _get_firmware_version(self) -> str:
        self._queue_info.queue.clear()
        self._write_line("f?")
        return int(self._get_and_check_info()[1])

    def _custom_info(self) -> list:
        self._queue_info.queue.clear()
        self._write_line(self.custom_info_str)
        return self._get_and_check_info(self.custom_info_size)

    def _cast_splitted_input_string(self, line: str):
        for i, data_type in enumerate(self._data_structure.values()):
            if type(line[i]) is not data_type:
                line[i] = data_type(line[i])

        return line

    #########################################
    ### Init methods
    def _init(self, serial_number, port, verb_freq):
        self._init_serial(serial_number, port)
        self._init_thread()
        self._init_data_structures()
        self._init_serial_data_structure()

        self._init_clear_buffers()
        self.start()
        self._init_get_device_info()

        self.set_verbose_freq(verb_freq)

    def _init_serial(self, serial_number: int, port: str):
        from naneos.partector import scan_for_serial_partector

        self._serial_number = serial_number
        self._port = port

        if self._serial_number:
            self._port = scan_for_serial_partector(self._serial_number)
        elif self._port is None:
            raise Exception("No serial number or port given!")

        self._ser = None
        if self._port:
            self._ser = serial.Serial(
                port=self._port,
                baudrate=self.SERIAL_BAUDRATE,
                timeout=self.SERIAL_TIMEOUT,
            )

        self._connected = False
        if self._ser:
            if self._ser.isOpen():
                self._connected = True
                logger.info(f"Connected to SN{self._serial_number} on {self._port}")

        self.set_verbose_freq(0)

    def _init_thread(self):
        Thread.__init__(self)
        self.name = "naneos-partector-thread"
        self.thread_event = Event()

    def _init_data_structures(self):
        self.custom_info_str = "0"
        self.custom_info_size = 0
        self._data_structure = None  # will be declared in child class
        self._queue = Queue(maxsize=self.SERIAL_QUEUE_MAXSIZE)
        self._queue_info = Queue(maxsize=self.SERIAL_INFO_QUEUE_MAXSIZE)

    @abstractmethod
    def _init_serial_data_structure(self):
        pass

    def _init_clear_buffers(self):
        if not self._connected:
            return

        time.sleep(10e-3)
        self._ser.reset_input_buffer()

    def _init_get_device_info(self):
        try:
            if self._serial_number is None:
                self._serial_number = self._get_serial_number_secure()
            self._firmware_version = self.get_firmware_version()
            logger.debug(f"Connected to SN{self._serial_number} on {self._ser.port}")
        except Exception:
            logger.warning("Could not get device info!")
            # port = self._ser.port
            # self.close()
            # raise ConnectionError(f"No partector2 on port {port}.")


if __name__ == "__main__":
    import time

    from naneos.partector import Partector2ProGarage

    partector = Partector2ProGarage(serial_number=8150)

    time.sleep(30)

    print(partector.get_data_pandas())
    partector.close(blocking=True)



import asyncio
import time

from bleak import BleakScanner
from bleak.backends.device import BLEDevice
from bleak.backends.scanner import AdvertisementData

from naneos.partector.partector1 import Partector1
from naneos.partector.scan_for_partector import scan_for_serial_partectors
from naneos.protobuf.protobuf import create_Partector1_entry


def simple_callback(device: BLEDevice, advertisement_data: AdvertisementData):
    if device.name != "P2":
        return

    print(f"Device: {device.name} ({device.address})")

    # create a bytearray from the manufacturer data key
    tmp = list(advertisement_data.manufacturer_data.keys())[0]
    p2AdvData = bytearray(tmp.to_bytes(2, byteorder="little"))
    # extend the bytearray with the manufacturer data value
    p2AdvData.extend(list(advertisement_data.manufacturer_data.values())[0])

    if len(p2AdvData) > 22:  # that means it has a scan response
        print(chr(p2AdvData[22]))

    sn = int(int(p2AdvData[15]) + (int(p2AdvData[16]) << 8))
    print(f"SN: {sn}")

    ldsa = (
        int(p2AdvData[1]) + (int(p2AdvData[2]) << 8) + (int(p2AdvData[3]) << 16)
    ) / 100.0
    print(f"LDSA: {ldsa}")


async def scanBle():
    # BleakScanner that only listens for "P2" devices

    scanner = BleakScanner(simple_callback, cb=dict(use_bdaddr=True))

    await scanner.start()
    await asyncio.sleep(2.0)
    await scanner.stop()


def scanWire():
    devs = scan_for_serial_partectors()
    print(devs)


def readP1():
    devs = scan_for_serial_partectors()
    devs = devs["P1"]
    print(devs)

    p1 = Partector1(list(devs.values())[0])

    time.sleep(5)

    # print(p1.get_data_list())
    df = p1.get_data_pandas()
    print(df)

    df.to_pickle("p1.pkl")

    p1.close()

    create_Partector1_entry(df)


if __name__ == "__main__":
    # asyncio.run(scanBle())
    # scanWire()
    readP1()



import time

from naneos.iotweb import P1uploadThread
from naneos.partector import scan_for_serial_partectors


def main():
    upload_thread = P1uploadThread()
    time.sleep(0.2)

    # wait for keyboard interrupt
    try:
        while True:
            time.sleep(1)
            connected_ports = upload_thread.get_connected_devices_ports()
            devs = scan_for_serial_partectors(sn_exclude=connected_ports)["P1"]
            # print(devs)

            # add all devices to the queue
            for v in devs.values():
                print(f"Adding {v} to queue")
                upload_thread.connect_queue.put(v)
    except KeyboardInterrupt:
        pass

    # stop the thread
    upload_thread.event.clear()
    upload_thread.join()


if __name__ == "__main__":
    main()



import datetime

# v295
PARTECTOR2_DATA_STRUCTURE = {
    "dateTime": datetime.datetime,
    "runtime_min": float,
    "idiff_global": float,
    "ucor_global": int,
    "hiresADC1": float,
    "hiresADC2": float,
    "EM_amplitude1": float,
    "EM_amplitude2": float,
    "T": float,
    "RHcorr": int,
    "device_status": int,
    "deposition_voltage": int,
    "batt_voltage": float,
    "flow_from_dp": float,
    "LDSA": float,
    "diameter": float,
    "number": int,
    "dP": int,
    "P_average": float,
}

# v302
PARTECTOR2_PRO_DATA_STRUCTURE = {
    "dateTime": datetime.datetime,
    "runtime_min": float,
    "number": int,
    "diameter": float,
    "LDSA": float,
    "surface": float,  # not existing in protobuf
    "particle_mass": float,
    "sigma": float,  # not existing in protobuf
    "idiff_global": float,
    "ucor_global": int,
    "deposition_voltage": int,
    "T": float,
    "RHcorr": int,
    "P_average": float,
    "flow_from_dp": float,
    "batt_voltage": float,
    "pump_current": float,  # not existing in protobuf
    "device_status": int,
    "pump_pwm": int,  # not existing in protobuf
    "steps": int,  # not existing in protobuf
    "particle_number_10nm": int,
    "particle_number_16nm": int,
    "particle_number_26nm": int,
    "particle_number_43nm": int,
    "particle_number_70nm": int,
    "particle_number_114nm": int,
    "particle_number_185nm": int,
    "particle_number_300nm": int,
    "current_0": float,  # not existing in protobuf
    "current_1": float,  # not existing in protobuf
    "current_2": float,  # not existing in protobuf
    "current_3": float,  # not existing in protobuf
    "current_4": float,  # not existing in protobuf
}

# same as PARTECTOR2_PRO_DATA_STRUCTURE but with cs_status at the end
PARTECTOR2_PRO_GARAGE_DATA_STRUCTURE = PARTECTOR2_PRO_DATA_STRUCTURE.copy()
PARTECTOR2_PRO_GARAGE_DATA_STRUCTURE["cs_status"] = int


PARTECTOR1_DATA_STRUCTURE = {
    "dateTime": datetime.datetime,
    "runtime_min": float,
    "batt_voltage": float,
    "idiff_global": float,
    "ucor_global": float,
    "EM": float,
    "DAC": float,
    "HVon": int,
    "idiffset": float,
    "flow": float,
    "LDSA": float,
    "T": float,
    "RHcorr": float,
    "device_status": int,
    # "phase_angle": float,
}



from datetime import datetime, timezone

from naneos.logger.custom_logger import get_naneos_logger
from naneos.partector.blueprints._data_structure import (
    PARTECTOR2_PRO_GARAGE_DATA_STRUCTURE,
)
from naneos.partector.blueprints._partector_blueprint import PartectorBluePrint

logger = get_naneos_logger(__name__)


class Partector2ProGarage(PartectorBluePrint):
    CS_OFF = 0
    CS_ON = 1
    CS_UNKNOWN = -1

    def __init__(
        self, serial_number: int = None, port: str = None, verb_freq: int = 1, **kwargs
    ):
        self._catalyst_state = self.CS_UNKNOWN
        self._auto_mode = True

        self._callback_catalyst = kwargs.get("callback_catalyst", None)
        super().__init__(serial_number, port, verb_freq)

    def _get_and_check_info(self, expected_length: int = 3) -> list:
        return super()._get_and_check_info(expected_length)

    def _init_serial_data_structure(self):
        self._data_structure = PARTECTOR2_PRO_GARAGE_DATA_STRUCTURE

    def _set_verbose_freq(self, freq: int):
        if freq == 0:
            self._write_line("X0000!")
        else:
            self._write_line("X0006!")

    def set_catalyst_state(self, state: str):
        """Sets the catalyst state to on, off or auto."""
        if not self._connected:
            return

        if state == "on":
            self._write_line("CSon!")
            self._cs_state = self.CS_ON
            self._auto_mode = False
        elif state == "off":
            self._write_line("CSoff!")
            self._cs_state = self.CS_OFF
            self._auto_mode = False
        elif state == "auto":
            self._write_line("CSauto!")
            self._auto_mode = True
        else:
            logger.warning(f"Unknown catalyst state: {state} -> nothing done.")
            return

        logger.info(f"Catalyst state set to {state}.")

    def _serial_reading_routine(self):
        line = self._read_line()

        if not line or line == "":
            return

        # check if line contains !CS_on or !CS_off and remove it from line
        if "!CS_on" in line:
            line = line.replace("!CS_on", "")
            self._callback_catalyst(True)
            self._put_line_to_queue(line, True)
            self._catalyst_state = self.CS_ON
        elif "!CS_off" in line:
            line = line.replace("!CS_off", "")
            self._callback_catalyst(False)
            self._put_line_to_queue(line, False)
            self._catalyst_state = self.CS_OFF
        else:
            self._put_line_to_queue(line)

    def _put_line_to_queue(self, line: str, cs_command: bool = None):
        data = [datetime.now(tz=timezone.utc)] + line.split("\t")
        data = data + [self._catalyst_state]

        if len(data) != len(self._data_structure):
            self._put_to_info_queue(data)
            return

        # removes the size dist from all the measurements that are not wanted
        if cs_command is None and self._auto_mode:
            data[20:28] = [0] * 8

        self._put_to_data_queue(data)

    def _put_to_info_queue(self, data: list):
        if self._queue_info.full():
            self._queue_info.get()
        self._queue_info.put(data)

    def _put_to_data_queue(self, data: list):
        if self._queue.full():
            self._queue.get()
        self._queue.put(data)


if __name__ == "__main__":
    import time

    def test_callback(state: bool):
        print("Catalyst state: {}".format(state))

    p2 = Partector2ProGarage(test_callback, "/dev/tty.usbmodemDOSEMet_1")

    # print(p2.write_line("v?", 1))
    time.sleep(3)
    print(p2.get_data_pandas()["cs_status"])

    print("Closing...")
    p2.close(blocking=True)
    print("Closed!")



import datetime as dt
import os

from naneos.iotweb import download_from_iotweb

token = os.getenv("IOT_GUEST_TOKEN", None)
if token is None:
    raise ValueError("No token found in your environment")

start = dt.datetime(2023, 11, 25)
stop = dt.datetime(2023, 11, 27)
serial_number = "8134"
name = "iot_guest"

df = download_from_iotweb(name, serial_number, start, stop, token)
print(df)



import asyncio

from bleak import BleakClient, BleakScanner

# Detecting devices
# Connecting devices
# Removing devices when disconnected


async def connect(address):
    async with BleakClient(address) as client:
        for service in client.services:
            print(f"[Service] {service}")

            for characteristic in service.characteristics:
                print(f"[Characteristic] {characteristic}")

                # value = await client.read_gatt_char(characteristic.uuid)
                # print(f"[Value] {value}")


async def discover():
    # filter for device name P2
    device1 = await BleakScanner.find_device_by_name("P2", timeout=4.0)
    await connect(device1.address)
    # print(device1)

    # device2 = await BleakScanner.find_device_by_name("P2", timeout=4.0)
    # print(device2)


async def scan():
    stop_event = asyncio.Event()

    def scanner_callback(device, advertisement_data):
        print(device, advertisement_data)

    # filter for name P2
    async with BleakScanner(scanner_callback) as scanner:
        await stop_event.wait()


if __name__ == "__main__":
    asyncio.run(discover())
